@@ -15,6 +15,74 @@ use serde::ser::{Serialize, Serializer};
 use super::ext::*;
 use super::{error, Complex, Result};
 
+/// The [`NumberType`] of the product of an [`Array`] with the given `array_dtype`.
+pub fn product_dtype(array_dtype: NumberType) -> NumberType {
+    use {ComplexType as CT, FloatType as FT, IntType as IT, NumberType as NT, UIntType as UT};
+
+    match array_dtype {
+        NT::Bool => ArrayExt::<bool>::product_dtype(),
+        NT::Complex(ct) => match ct {
+            CT::C32 => ArrayExt::<Complex<f32>>::product_dtype(),
+            CT::C64 => ArrayExt::<Complex<f64>>::product_dtype(),
+            CT::Complex => ArrayExt::<Complex<f64>>::product_dtype(),
+        },
+        NT::Float(ft) => match ft {
+            FT::F32 => ArrayExt::<f32>::product_dtype(),
+            FT::F64 => ArrayExt::<f64>::product_dtype(),
+            FT::Float => ArrayExt::<f64>::product_dtype(),
+        },
+        NT::Int(it) => match it {
+            IT::I8 => ArrayExt::<i16>::product_dtype(),
+            IT::I16 => ArrayExt::<i16>::product_dtype(),
+            IT::I32 => ArrayExt::<i32>::product_dtype(),
+            IT::I64 => ArrayExt::<i64>::product_dtype(),
+            IT::Int => ArrayExt::<i64>::product_dtype(),
+        },
+        NT::UInt(ut) => match ut {
+            UT::U8 => ArrayExt::<u8>::product_dtype(),
+            UT::U16 => ArrayExt::<u16>::product_dtype(),
+            UT::U32 => ArrayExt::<u32>::product_dtype(),
+            UT::U64 => ArrayExt::<u64>::product_dtype(),
+            UT::UInt => ArrayExt::<u64>::product_dtype(),
+        },
+        NT::Number => ArrayExt::<f64>::product_dtype(),
+    }
+}
+
+/// The [`NumberType`] of the sum of an [`Array`] with the given `array_dtype`.
+pub fn sum_dtype(array_dtype: NumberType) -> NumberType {
+    use {ComplexType as CT, FloatType as FT, IntType as IT, NumberType as NT, UIntType as UT};
+
+    match array_dtype {
+        NT::Bool => ArrayExt::<bool>::sum_dtype(),
+        NT::Complex(ct) => match ct {
+            CT::C32 => ArrayExt::<Complex<f32>>::sum_dtype(),
+            CT::C64 => ArrayExt::<Complex<f64>>::sum_dtype(),
+            CT::Complex => ArrayExt::<Complex<f64>>::sum_dtype(),
+        },
+        NT::Float(ft) => match ft {
+            FT::F32 => ArrayExt::<f32>::sum_dtype(),
+            FT::F64 => ArrayExt::<f64>::sum_dtype(),
+            FT::Float => ArrayExt::<f64>::sum_dtype(),
+        },
+        NT::Int(it) => match it {
+            IT::I8 => ArrayExt::<i16>::sum_dtype(),
+            IT::I16 => ArrayExt::<i16>::sum_dtype(),
+            IT::I32 => ArrayExt::<i32>::sum_dtype(),
+            IT::I64 => ArrayExt::<i64>::sum_dtype(),
+            IT::Int => ArrayExt::<i64>::sum_dtype(),
+        },
+        NT::UInt(ut) => match ut {
+            UT::U8 => ArrayExt::<u8>::sum_dtype(),
+            UT::U16 => ArrayExt::<u16>::sum_dtype(),
+            UT::U32 => ArrayExt::<u32>::sum_dtype(),
+            UT::U64 => ArrayExt::<u64>::sum_dtype(),
+            UT::UInt => ArrayExt::<u64>::sum_dtype(),
+        },
+        NT::Number => ArrayExt::<f64>::sum_dtype(),
+    }
+}
+
 /// A generic one-dimensional array which supports all variants of [`NumberType`].
 #[derive(Clone)]
 pub enum Array {
@@ -115,30 +183,28 @@ impl Array {
 
     /// The [`NumberType`] of this `Array`.
     pub fn dtype(&self) -> NumberType {
+        use number_general::DType;
         use Array::*;
+
         match self {
-            Bool(_) => NumberType::Bool,
-            C32(_) => ComplexType::C32.into(),
-            C64(_) => ComplexType::C64.into(),
-            F32(_) => FloatType::F32.into(),
-            F64(_) => FloatType::F64.into(),
-            I16(_) => IntType::I16.into(),
-            I32(_) => IntType::I32.into(),
-            I64(_) => IntType::I64.into(),
-            U8(_) => UIntType::U8.into(),
-            U16(_) => UIntType::U16.into(),
-            U32(_) => UIntType::U32.into(),
-            U64(_) => UIntType::U64.into(),
+            Bool(_) => bool::dtype(),
+            C32(_) => Complex::<f32>::dtype(),
+            C64(_) => Complex::<f64>::dtype(),
+            F32(_) => f32::dtype(),
+            F64(_) => f64::dtype(),
+            I16(_) => i16::dtype(),
+            I32(_) => i32::dtype(),
+            I64(_) => i64::dtype(),
+            U8(_) => u64::dtype(),
+            U16(_) => u64::dtype(),
+            U32(_) => u64::dtype(),
+            U64(_) => u64::dtype(),
         }
     }
 
     /// Cast into an `Array` of a different `NumberType`.
     pub fn cast_into(&self, dtype: NumberType) -> Array {
-        use ComplexType as CT;
-        use FloatType as FT;
-        use IntType as IT;
-        use NumberType as NT;
-        use UIntType as UT;
+        use {ComplexType as CT, FloatType as FT, IntType as IT, NumberType as NT, UIntType as UT};
 
         match dtype {
             NT::Bool => Self::Bool(self.type_cast()),
@@ -650,7 +716,7 @@ impl Array {
     pub fn product(&self) -> Number {
         use Array::*;
         match self {
-            Bool(b) => Number::UInt(b.product().into()),
+            Bool(b) => Number::Bool(b.product().into()),
             C32(c) => Number::Complex(c.product().into()),
             C64(c) => Number::Complex(c.product().into()),
             F32(f) => Number::Float(f.product().into()),
@@ -1427,11 +1493,7 @@ where
 
 impl From<Vec<Number>> for Array {
     fn from(elements: Vec<Number>) -> Self {
-        use ComplexType as CT;
-        use FloatType as FT;
-        use IntType as IT;
-        use NumberType as NT;
-        use UIntType as UT;
+        use {ComplexType as CT, FloatType as FT, IntType as IT, NumberType as NT, UIntType as UT};
 
         let dtype = elements.iter().map(|n| n.class()).fold(NT::Bool, Ord::max);
 
@@ -1760,6 +1822,13 @@ mod tests {
         assert_eq!(&a * &b, [5, 8, 9][..].into());
     }
 
+    #[test]
+    fn test_mul_const() {
+        let a: Array = [1, 2, 3][..].into();
+        let b: Number = 2f32.into();
+        assert_eq!(&a * b, [2.0, 4.0, 6.0][..].into());
+    }
+
     #[test]
     fn test_mul_float() {
         let a: Array = [1, 2, 3][..].into();
